/*
 *          Copyright (c) mjh-EDV Beratung, 1996-1999
 *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12
 *          Tel +49 6102 328279 - Fax +49 6102 328278
 *                Email info@mjh.teddy-net.com
 *
 *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *   $Id: messages.c,v 1.5 1999/08/19 05:54:12 jordan Exp $
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "common-stuff.h"

#ifdef USE_PTHREADS
#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif
#endif

#include "messages.h"
#include "iostream.h"

#define DUMP_MUTEX

/* --------------------------------------------------------------------------- *
 *               private defs and data and structures                          *
 * --------------------------------------------------------------------------- */

#define __null__(f) f(0,"")
#define errnum_index(sym)     (errnum (sym) - errnum (__null__))

#define __plus_one__(f)       + 1
#define generic_list_DIM      (0 PEKS_GENERIC_LIST (__plus_one__))

/* --------------------------------------------------------------------------- *
 *                  private variables                                          *
 * --------------------------------------------------------------------------- */

static char generic_buffer [generic_list_DIM] [200] ;
static unsigned current_inx = 0 ;

/* --------------------------------------------------------------------------- *
 *                  private functions                                          *
 * --------------------------------------------------------------------------- */

#define SYSNAME "messages"
#include "peks-sema.h"

/* --------------------------------------------------------------------------- *
 *                  public functions                                           *
 * --------------------------------------------------------------------------- */

#ifdef USE_PTHREADS
void messages_sema_create (void *attr) { __sema_create (attr); }
void messages_sema_destroy      (void) { __sema_destroy    (); }
#endif

unsigned
peks_errnum
  (const char *s)
{
  unsigned inx ;

  if (s == 0)
    return -1;

  __enter_lock_semaphore () ;  /* Posix threads support */

  /* get current buffer index */ 
  inx = current_inx ;
  if (++ current_inx >= generic_list_DIM)
    current_inx = 0 ;

  /* store message */
  strncpy (generic_buffer [inx], s, sizeof (generic_buffer [0])) ;
  generic_buffer [inx] [sizeof (generic_buffer [0]) - 1] = '\0' ;

  __release_lock_semaphore () ;  /* Posix threads support */

  /* return a message number */
  return errnum (__null__) + (inx + 1);
}



char *
peks_strerr
  (unsigned n)
{
  extern int sys_nerr ;
  char *s = 0 ;
  
# define do_case(sym) \
       case errnum (sym): s = generic_buffer [errnum_index (sym) - 1]; break;

  __enter_lock_semaphore () ;  /* Posix threads support */
  switch (n) {
    PEKS_GENERIC_LIST (do_case);
  } /* switch */
  __release_lock_semaphore () ;  /* Posix threads support */

  /* note: this is not totally failproof ... */
  if (s != 0)
    return s;

# undef  do_case



# define do_case(sym) \
       case errnum (sym): return errstr (sym) ;

  switch (n) {
    PEKS_ERRLIST1 (do_case) ;
    PEKS_ERRLIST2 (do_case) ;
    PEKS_ERRLIST3 (do_case) ;
    PEKS_ERRLIST4 (do_case) ;
  } /* switch */

# undef do_case

  if (n < sys_nerr)
    return strerror (n) ;

  return "unspecified peks error" ;
}
