/*
 *          Copyright (c) mjh-EDV Beratung, 1996-1999
 *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12
 *          Tel +49 6102 328279 - Fax +49 6102 328278
 *                Email info@mjh.teddy-net.com
 *
 *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *   $Id: us_export_wizard.c,v 1.5 1999/08/19 05:54:13 jordan Exp $
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "common-stuff.h"

#ifndef HAVE_RAND
#define  rand()  random ()
#define srand() srandom ()
#endif

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

main (int argc, char **argv) {
  
  int keybits = argc > 1 ? atoi (argv [1]) : 9999 ;
  int keylen  = keybits > 5 ? keybits/8 : 5 ;
  int alen    = 20 ;

  keybits = keylen * 8 ;

  printf ("/*\n");
  printf (" *          Copyright (c) mjh-EDV Beratung, 1996-1999\n");
  printf (" *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12\n");
  printf (" *          Tel +49 6102 328279 - Fax +49 6102 328278\n");
  printf (" *                Email info@mjh.teddy-net.com\n");
  printf (" *\n");
  printf (" *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>\n");
  printf (" *\n");
  printf (" *   $Id: us_export_wizard.c,v 1.5 1999/08/19 05:54:13 jordan Exp $\n");
  printf (" *\n");
  printf (" *   This library is free software; you can redistribute it and/or\n");
  printf (" *   modify it under the terms of the GNU Library General Public\n");
  printf (" *   License as published by the Free Software Foundation; either\n");
  printf (" *   version 2 of the License, or (at your option) any later version.\n");
  printf (" *\n");
  printf (" *   This library is distributed in the hope that it will be useful,\n");
  printf (" *   but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
  printf (" *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n");
  printf (" *   Library General Public License for more details.\n");
  printf (" *\n");
  printf (" *   You should have received a copy of the GNU Library General Public\n");
  printf (" *   License along with this library; if not, write to the Free\n");
  printf (" *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n");
  printf (" *\n");
  printf (" *   key weakener - make a cipher key US export laws conformant ...\n");
  printf (" */\n\n");
  printf ("#ifndef __US_EXPORT_WIZARD__\n");
  printf ("#define __US_EXPORT_WIZARD__\n\n");

  if (keylen > 16) {

    printf ("/* We do not need a key weakener, here. */\n");
    printf ("#undef STATIC_KEY_POSTFIX\n");
    printf ("#undef TRUE_KEY_BITS\n");
    printf ("#undef TRUE_KEY_BYTES\n");
    printf ("#undef US_EXPORT_MAGICIAN\n");

    printf ("\n");
    
  } else {

    unsigned char fp [8] ;
    int n, j, k ;

#ifdef HAVE_GETTIMEOFDAY
    struct timeval tv ; 
    gettimeofday2 (&tv, 0);
    srand (tv.tv_usec);
#else
    srand (time (0));
#endif
 
    printf ("#warning Restricting key lengths to %u bits\n\n", keybits);

    printf ("#define TRUE_KEY_BITS  %3u\n", keybits);
    printf ("#define TRUE_KEY_BYTES %3u\n", keylen);

    printf ("#define STATIC_KEY_POSTFIX {");
    for (n = j = k = 0; n < alen; n ++, j ++, k ++) {
      /* calculate an integer smaller than 256.0 - 1/(RAND_MAX-1.0) */
      unsigned char r = (256.0 * rand () / (RAND_MAX+1.0)) ;
      if (j > 6) { j = 0 ; printf ("\\\n                            "); }
      if (k < sizeof (fp)) fp [k] = r ;
      printf (" 0x%02x,", r);
    }
    printf ("    0 }\n\n");
    
    printf ("#define US_EXPORT_BITS        \"%u\"\n", keybits);
    printf ("#define US_EXPORT_FINGERPRINT \"");
    for (j = k = 0; k < sizeof (fp); j ++, k ++) {
      if (j >= (sizeof (fp)/2)) { j = 0 ; printf (" "); }
      if (k) printf (" ");
      printf ("%02x", fp [k]);
    }
    printf ("\"\n\n");

    printf ("#define US_EXPORT_MAGICIAN( key, len) \t\t\t\t\\\n");
    printf ("\tif ((len) > %u) { \t\t\t\t\t\\\n", keylen);
    printf ("\t\tchar p [] = STATIC_KEY_POSTFIX, i, j; \t\t\\\n");
    printf ("\t\tfor (i = 5, j = 0; i < (len); i ++, j ++) { \t\\\n");
    printf ("\t\t\tif (j >= %u) j = 0 ; \t\t\t\\\n", alen);
    printf ("\t\t\t(key) [i] = p [j]; \t\t\t\\\n");
    printf ("\t\t} \t\t\t\t\t\t\\\n");
    printf ("\t}\n\n");

  }
  printf ("#endif /* __US_EXPORT_WIZARD__*/\n");
  exit (0) ;
}
